function jack_cycle(filename, Quantity, ...
	minB, maxB, stepB, discard_first_col, invertB, log_x_axis)
%   function jack_cycle(filename, Quantity, ...
%               minB, maxB, stepB, discard_first_col, invertB, ...
%               log_x_axis)
%
%   Loads a datafile, performs a cycle of jackkife analyses
%   over various bin-sizes by applying a function on it,
%   prints the average and plots the bin-size dependence of the error.
%
%   No return parameters.
%   
%   Input parameters:
%      filename          a valid filename of the (ascii-table) file to load.
%      Quantity          function handle of the f(...) to apply: must have
%                        as input a vector same as the input-file row
%                        (without the leading index-column, if any)
%                        and as output one number (or more, packed in a vector).
%     [minB]             minimum bin-size to start the cycle with.
%     [maxB]             maximum bin-size in the cycle.
%     [stepB]            step in increasing the bin size.
%    [discard_first_col] if nonzero, the first column in the file is assumed
%                        to contain indices 1,2,... and is discarded.
%    [invertB]           if nonzero, the plot has 1/bin-size
%                        as x-axis instead of bin-size
%    [log_x_axis]        if nonzero, sets x-axis to be in log-scale
%
%  Only the first two arguments are mandatory; the other default to:
%     maxB               -> Nsamples/25
%     minB               -> min( 50, maxB/10 )        [made integer]
%     stepB              -> max( 3 , (maxB-minB)/80 ) [made integer]
%     discard_first_col  -> 1
%     invertB            -> 1
%     log_x_axis         -> 0

% Handling defaults and bad values, part I (i.e. no binning params)
% Not so important, skip ahead for the juicy stuff.
% Here use of 'exist' is made. Note that also the variable names are
% strings here. Also 'isempty' is employed, and 'error' to raise exceptions.
if ~exist('filename','var')||isempty(filename)
	error('filename must be provided.');
endif
if ~exist('Quantity','var') % note 'Quantity' is also a variable for this check
	error('Quantity must be provided.');
endif
if ~exist('discard_first_col','var')
	discard_first_col=1;
endif
if ~exist('invertB','var')
	invertB=1;
endif
if ~exist('log_x_axis','var')
	log_x_axis=0;
endif

% load input datafile and, if requested, trash first column
data=load('-ascii',filename);
if discard_first_col
	data=data(:,2:end);
endif

% Handling defaults and bad values, part II:
% binsize-related things
if ~exist('maxB','var')||isempty(maxB)||(maxB>size(data,1))
	maxB = floor(size(data,1)/25);
endif
if ~exist('minB','var')||isempty(minB)
	minB = min(50,floor(maxB/10));
endif
if ~exist('stepB','var')||isempty(stepB)
	stepB = max(3,floor((maxB-minB)/80));
endif
if stepB<=1
	stepB=1;
endif

% Proceed with the body of the routine.
% Determine how many bin sizes to examine
Nbins = 1+floor((maxB-minB)/stepB);
% If necessary, adjust maxB to its rounded value.
maxB  = minB + (Nbins-1)*stepB;

% Some on-screen info. Note string concatenation at work.
disp(['Analysing ''' filename '''. Min/Max binsize = ' ...
	num2str(minB) '/' num2str(maxB) ', total ' num2str(Nbins) ...
	' choices with step ' num2str(stepB) '.' ]);

% Actual computation:
% Here it is useful to determine in advance n_out
%   (i.e. the number of scalars returned by 'Quantity').
% This is done with a dummy call on the mean value of data.
n_out=length(Quantity(mean(data,1)));
% store the various bin sizes in a vector specified through (min:step:max) notation
binsizes=(minB:stepB:maxB);
% preallocate results' tables for averages and errors:
values=zeros(Nbins,n_out);
errors=zeros(Nbins,n_out);
% cycle through binsizes and repeat the call to jackknife proper
for i_bsize = 1:Nbins
	bin_size = binsizes(i_bsize);
	% this line is the call to the 'jackknife engine'
	[values(i_bsize,:), errors(i_bsize,:)] = jackknife_faster(data,bin_size,@(x) Quantity(x));
	% Alternatively, slower:
	%    [values(i_bsize,:), errors(i_bsize,:)] = jackknife(data,bin_size,@(x) Quantity(x));
	% (go looking at the difference between the two implementations; there's a huge optimisation in the former).
endfor

% Now the plot of the errors against either bin-size or 1/bin-size.
% First let us check if there will be a plot-flood
% and ask the user if she wants to choke the machine.
if n_out>5
	% note the handy function 'input' for interaction with user
	max_plot = input (['Possible plotbomb detected: ' num2str(n_out) ...
		' plots will be produced. Please tell me how many plots ' ...
		'will actually be drawn and press Enter : ']);
	max_plot = min(n_out,max_plot);
else
	max_plot = n_out;
endif
% Actual plotting. First prepare the x-axis and its label according to the flag.
if invertB
	x_axis=1./binsizes;
	x_desc='1/B';
else
	x_axis=binsizes;
	x_desc='B';
endif
% Each output value is a separate plot:
% 'figure' will be used.
for i_plot = 1:max_plot
	figure % this opens a new figure for subsequent graphics commands
	if log_x_axis==0
		plot(x_axis, errors(:,i_plot), 'bo','linewidth',4);
	else
		semilogx(x_axis, errors(:,i_plot), 'bo','linewidth',4);
	endif
	% blue, with no lines and thick points 
	xlabel(x_desc);
	ylabel(['Error(der.obs ' num2str(i_plot) ')']);
	% make sure y-axis starts from zero for readability.
	% To do this, the function 'axis' returns a 4-uple,
	% whose 3th and 4th values are ymin and ymax. Alter them,
	% and again call 'axis' with an input to set the new values.
	limit = axis;
	limit(3)=0;
	axis(limit);
endfor

% Last, print on screen the average value.
% It should be irrelevant whether one displays
% just values(1,:) or mean(values,1), but for
% aesthetics we'll average all determinations
% (the difference is due to different truncations
%  of the whole set of input with different bin-sizes).
% Note that num2str handles vectors easily.
disp(['Average value = ' num2str(mean(values,1)) '.']);

endfunction
