function [out_average, out_error] = jackknife_faster(data, binsize, Quantity)
	%    [out_average, out_error] = jackknife_faster(data, binsize, Quantity)
	% Jackknife analysis with a nonlinear function of primary observables.
	%
	%    DOES THE SAME AS JACKKNIFE.M, BUT IS FASTER - SEE LINE 40
	%       (everything else is unchanged!)
	%
	%   'data' is a N x N_obs array of N_obs primary observables
	%        along a chain of length N (i.e. the 'p_alpha^i').
	%   'binsize' is a positive integer specifying the length of a bin
	%       (if there is a remainder at the end of the N data, some are lost).
	%   'Quantity' must be a function from a n_obs-uple of
	%       numbers to a n_out-uple of numbers (e.g. an eff. mass,
	%                m_eff = log(corr(t)/corr(t+1)) ,
	%       is a map: R^2 -> R) and formally must be a y = f(x),
    %       with x a n_obs-element vector and y a n_out-element vector.
	% Returns average value and error of function(primary observables).

	% assume 'data' is a rectangular matrix of size (n_samples x n_observables),
	% get these numbers (n_seq==N, n_obs==N_alpha)
	[n_seq n_obs]=size(data);
	% determine number of bins.
	% Note that the last fraction of a bin, if it exists
	% (i.e. if n_seq/binsize not integer), will be lost in the following.
	% (floor returns the highest integer right below the argument)
	n_bins = floor(n_seq/binsize);
	% pre-allocate the whole table 'pbar_alpha^b' (pbartable) by creating all-zeros.
	% (this speeds up things later)
	pbartable = zeros(n_bins,n_obs);
	% build binned averages of input data, there called: 'c_alpha^k'
	%    (see Eq.63 of hep-lat/0306017v4)
	
	% The full mean of the primary-obs is require to later get \overline{F}.
	% 'p_fullmean' contains the fully-averaged 'a_alpha^i' and still is a vector along alpha.
	% But here also discard off-bin entries, if any:
	p_fullmean = mean(data(1:(binsize*n_bins),:),1);
	
	% and actual computation of the binned-averages
	for b_bin=1:n_bins
		% This is the only difference compared to 'jackknife.m'.
		% Here some (potentially significant) speedup is achieved
		% by observing that the same binned-complementary-average
		% calculated in 'jackknife.m' can be obtained faster, provided
		% you already have the *full mean*, by taking:
		%   Nb*fullmean/(Nb-1) - sum_over_this_bin(p^m)/(B*(Nb-1))
		pbartable(b_bin,:)=( n_bins*p_fullmean - ... 
			mean(data( ((b_bin-1)*binsize+1):(b_bin*binsize),: ),1) ) / (n_bins-1) ;
		% This achieves, for long chains, substantial speed-ups, e.g.
		% for 'blue.dat' with the jack_cycle call
		%      jack_cycle('../Datafiles/blue.dat', @(x) log(x(1)/x(2)),10,3000,10,1,0,0)
		% yields a factor 11.5 in speed as compared to the simple 'jackknife.m'.
	endfor
	
	% Now for the derived observable.
	% First build 'f' (\overline{F}), that is f(full average of input observables).
	% This will be stored in 'out_average' (the output value of this routine)
	% and is obtained by calling the function handle 'Quantity' passed to this call:
	out_average = Quantity(p_fullmean);
	% Computation of the error on f.
	% Eq.65 of hep-lat/0306017v4 (jackknife error estimator) is now applied.
	% First, apply Quantity() row by row to the pbartable, finding the table of the 'f(pbar_alpha^b)'
	% Each call of 'Quantity' will have a n_obs-vector as input (one row of the pbartable).
	% There does not seem to be an equivalent to 'arrayfun' taking generic n_obs-uples as input,
	% so we have to resort to an explicit for loop (too bad!).
	% Again, preallocate the table of the 'f(pbar_alpha^b)' with zeros:
	% (note that, to find n_out, 'length' is used on a previous result: out_average will be
	%  a vector of length n_out, since it has been just filled by 'Quantity' itself)
	out_variance_table=zeros(n_bins,length(out_average));
	% row by row (i.e. bin by bin) the table is filled:
	for b_bin=1:n_bins
		out_variance_table(b_bin,:) = (Quantity(pbartable(b_bin,:)) - out_average ).^2 ;
		% ".^" is the element-wise square, as is to be done here
		% this is the k_bin-th contribution to Eq. 65.
	endfor
	% Final steps in getting the error of \overline{F}, as per Eq. 65:
	out_variance = ((n_bins-1)/(n_bins)) * sum( out_variance_table,1 );
	out_error = sqrt(out_variance);
	% both returns parameters (out_average, out_error) have been provided, end of the task.
endfunction
