% Program to use UWerrTexp by applying it to:
%   - either primary observable a_i
%   - or derived observable
%         (meff(a1,a2), a1+a2, a1-a2, a1*a2, a1/a2)
% on one of the six dataset files made available
% in '../Datafiles/'
% SETTINGS
startDataDir = '../Datafiles';


% Besides demonstrating the use of 'UWerrTexp' itself,
% this also serves the purpose of illustrating a few
% nice tricks that can be done with Octave.

% 1. get the list of datafiles available
%   this uses some filesystem functions
%   and string tokenisation, strtok
%   and is structured as to include subdirectories
remnant = genpath(startDataDir); % all subdirs in a string, separated by ":"
list={}; % initialise cell-array of all subdirs
while(~isempty(remnant))
	[newdir, remnant] = strtok(remnant, ':'); % take next token into 'newdir', reduce what's left
	list = {list{:} newdir}; % append newdir to 'list'
endwhile
% now each element list{i} is a dir with potentially datfiles - find them into a big list
fullfilelist=[];
for tdir=1:length(list)
	qfiles=dir([list{tdir} '/*.dat']); % qfiles will be an array of structures,
	% each with its field 'name'
	for tfile=1:length(qfiles)
		newelem.title = qfiles(tfile).name;
		newelem.fullpath = [list{tdir} '/' qfiles(tfile).name];
		fullfilelist = [fullfilelist newelem];
	endfor
endfor
% now each element in fullfilelist has a 'title' (nickname for it) and its full path relative to here.

% Ask the user which file she wants to load
printf ('  Available files:\n');
for ifile=1:length(fullfilelist)
	printf('    %2u.  %s\n', ifile , fullfilelist(ifile).title);
endfor
fchoice=-1;
while(fchoice<0 || fchoice>length(fullfilelist))
	fchoice=input('  Type your fchoice (zero to end) : ');
endwhile
if fchoice==0
	return;
endif

% now load the file, and discard the first column (assume it's 1,2,3 ...)
data = load('-ascii',fullfilelist(fchoice).fullpath);
data(:,1)=[];
numCols = size(data,2);

% 'database' of available observables: their symbol, their name,
%               the number of columns needed, their actual value
%  Here all those will be each a cell-array (it could have been, for
%  more flexibility, a cell-array of structures each with all info in it)
obs_symbols = {'1' '2' 'm' '+' '-' '*' '/'};
obs_names = {'p_1' 'p_2' 'meff(p_1,p_2)' 'p_1+p_2' 'p_1-p_2' 'p_1*p_2' 'p_1/p_2'};
obs_ncols = {1 2 2 2 2 2 2};
obs_qties = {1 2 @(x)log(x(1)/x(2)) @(x)(x(1)+x(2)) @(x)(x(1)-x(2)) ... 
	 @(x)(x(1)*x(2))  @(x)(x(1)/x(2))}; % a cell-array with some numbers and some function-handles. Why not?

% now ask the user which operation she wants on the primary obs:
printf ('  Available observables:\n');
for iop=1:length(obs_symbols)
	printf('    ''%s'' for %s\n', obs_symbols{iop}, obs_names{iop});
endfor
ochoice='~';
while(nnz(strcmp(ochoice,obs_symbols))==0 && ~isempty(ochoice))
	ochoice=input ('  Type your choice (empty to end) : ', 's');
endwhile
if isempty(ochoice)
	return;
endif
% convert ochoice to a number
op_idx=-1;
for ich=1:length(obs_symbols)
	if strcmp(ochoice,obs_symbols{ich})
		op_idx=ich;
		break;
	endif
endfor
if op_idx==-1
	error('The impossible happened.');
end
if numCols < obs_ncols{op_idx}
	error('Not enough columns in datafile.');
endif

% TEXP INFORMATION here
tauexp=0;
tauexp = input ( '  If you know it, type tau_exp, otherwise leave blank : ' );
if isempty(tauexp)
	tauexp=0;
endif

% suppress plots?
if yes_or_no ('  Suppress plots? ')
	Name=0;
else
	Name = obs_names{op_idx};
endif

% now all has been chosen by the user to finally call UWerrTexp. Do it.
Nrep = size(data,1);
parms(1)=1.5; % or some similar choice, or let the user pick one, etc. This is 'S'

% the previously obtained tauexp is here put into parms
parms(2) = tauexp;
% in this script, parms(4, ... 6) are left to their defaults,
% while parms(3) = Nsigma is set to 1 (default would be 3).
parms(3) = 1;

% the call to UWerrTexp, with its full output retained.
[val,err,derr,tau,dtau,Q,Wopt,gamma,drho] = UWerrTexp(data,parms,Nrep,Name,obs_qties{op_idx});

% prepare a nice formatting for the output
disp(' -----------------------------------');
disp(['  Value, err, deltaErr = ' num2str(val) ', ' num2str(err) ', ' num2str(derr)]);
disp(['  Tauint, errTauint    = ' num2str(tau) ', ' num2str(dtau)]);
if isempty(Q) % which will be the case here, since no replica are set up
	disp(['  Woptimal             = ' num2str(Wopt)]);
else
	disp(['  Q, Woptimal          = ' num2str(Q) ', ' num2str(Wopt)]);
endif
disp(' -----------------------------------');

if yes_or_no('  Do you want to dump Gamma(t) to file ? ')
	% write the table to a datafile
	fulltable = [(1:length(gamma))' gamma (drho * gamma(1))]; % remember drho is the error on the gamma(t)/gamma(0) !
	filename = ['gammaTable_' fullfilelist(fchoice).title];
	save('-ascii',filename,'fulltable');
	disp(['    Gamma table saved to ''' filename '''.']);
endif
