function fitparams = do_polyfit(xlist, ylist, Wmatrix, degree)
    % Polynomial fit to data (xlist,ylist)
    % Wmatrix is a weight matrix, e.g. C^(-1) or diag(1/sigma^2)
    %              or =identity (default)
    % degree (default=1) means the returned vector will have degree+1 values,
    % which are the cofficients b0, b1, ... b_{degree} for:
    %
    %    y = sum_{i=0}^{degree} b_i x^i
    %

    % preliminarly, check if the vectors passed are rows; if so, flip them.
    [n1 n2]=size(xlist);
    if n1==1
		xlist=xlist';
    endif
    [n1 n2]=size(ylist);
    if n1==1
		ylist=ylist';
    endif

    % counter
    nPoints = length(xlist);
    
    % defaults
    if ~exist('Wmatrix','var') || isempty(Wmatrix)
        Wmatrix = eye(nPoints);
    endif
    if ~exist('degree','var') || isempty(degree)
        degree = 1;
    endif
    
    % counter
    nParams = degree + 1;
    
    % prepare matrices
    X = zeros(nPoints,nParams);
    for irow=1:nPoints;
        for icol=1:nParams;
            X(irow,icol) = (xlist(irow))^(icol-1);
        endfor
    endfor
    Q = X' * Wmatrix;
    Z = Q * X;
    
    % now solve for the fit parameters:
    % One would do this:
    %     fitparams = (inv(Z)*Q)*ylist; or, even better:
    fitparams = Z\(Q*ylist);
    % but note: there might be numerical stability issues with the above inverse.
    % If the scales are very peculiar (e.g. difference among the x values much
    % smaller than their average), try shifting the x values or multiplying
    % them by a factor - and adjust the subsequent handling of the results.
    %
    % This is precisely the case for the 1/kappa business done here, so a trick can be used
    % (there's a function in Octave for 'conjugate gradients squared'
    %  method of solving lin.systems) instead of the above.
    % Arguments are: A,b,tolerance,max_iterations, returned is X such that AX=y.
    %      fitparams = cgs(Z,Q*ylist,1e-12,10);
endfunction
