function resultingSpacing = find_spacing(MF_pairs, kappalist, ratio_phys)
% resultingSpacing = find_spacing(MF_pairs, ratio_phys)
%
% Given pairs of (a*m_pi), (a*f_pi) for various ensembles
% (whose kappa is also given),
% performs first an extrapolation in (m_pi/f_pi)^2 to find the physical
% kappa, then uses it to extrapolate f_pi and set the scale with it.
% (Note: it is assumed that all ensembles share the same beta).
%
% INPUT PARAMETERS:
%    - MF_pairs is a vector of 2*N entries (i.e. N ensembles),
%               structured as: [M_1 F_1 M_2 F_2 ... M_N F_N]
%               (where M = a*m_pi, F=a*f_pi)
%    - kappalist is a vector of the corresponding N values
%               of the quark parameter kappa (one per ensemble).
%    - ratio_phys is the physical ratio of (m_pi/f_pi)^2 defining
%               the physical point. In our universe, this is
%               1.0686232 (default).

% deal with defaults
if ~exist('ratio_phys','var') || isempty(ratio_phys)
	ratio_phys=1.0686232;
endif

% now proceed. All fits here are done 'with no error-bars',
% meaning: all points have the same weight,
%    i.e. weight matrix = W = diag(1,1,...).

% first, build the table of (M/F)^2 for each ensemble
nPoints = length(kappalist);
ratioList = zeros(nPoints,1);
for ipt=1:nPoints
	ratioList(ipt) = ((MF_pairs(2*ipt-1)/MF_pairs(2*ipt)))^2;
endfor
invkappa=1./kappalist';
% now do the first-order-polynomial fit
% assume (y=invkappa, x=ratioList) that
%   y = a0 + a1*x
% then the fit can be done exactly with matrix algebra.
% For this, it is convenient to pack this
% into another function for generic polynomial fit
Wmatrix = eye(nPoints);
fitparams1 = do_polyfit(invkappa, ratioList, Wmatrix, 1);
% (do_polyfit is written to accept a weight-matrix.
%  Here, it is lazily left to the identity - that is, each point counts the same)
% (*) An alternative is to pre-evaluate the individual errors
%     on the ratios, ensemble by ensemble, and build W=diag(1/errors^2).
% (*) In this case it would be even better to give more weight to
%     ensembles closer to the physical point, such as giag(distance of ratio to ratio_phys):
%          Wmatrix = diag(1./(ratioList-ratio_phys)) ;

% now 'fitparams1' contains [a0 a1] for the 1st-order polynomial
% describing ratio=ratio(1/kappa). Solve ratio=ratio_phys to get kappa_phys.
invkappaphys = (ratio_phys-fitparams1(1))/fitparams1(2);

% second step: fit (linearly) this time a*f_pi as a function of 1/kappa,
% and use the invkappaphys obtained above to extrapolate this and get a*f_pi(phys.point).
% build the subvector of 'F' values only
flist = MF_pairs(2:2:end)';
% ready to go with the linear fit. Same weights as above.
fitparams2 = do_polyfit(invkappa, flist, Wmatrix, 1);
% that means, y = b0 + b1*(1/k)
% and F(phys.point) is going to be b0 + b1*(invkappaphys)
af_phys = fitparams2(1) + fitparams2(2)*invkappaphys;

% Transform the extrapolated af_phys to a lattice spacing.
% conversion ratio from (a*fpi) to lattice spacing in fermi
afpi_to_fermi = (197.33/130.4); % (hbar*c/f_pi) = 197.33 Mev*fm / 130.4 Mev

resultingSpacing = af_phys*afpi_to_fermi ;

endfunction
