function [val, err, tauintlist] = multiUWerrTexp(funhandle, inData, labels, tauExpList)
	% multi-ensemble UWerrTexp.
	% Singles out ensemble-by-ensemble contributions,
	% invokes repeatedly UWerrTexp and merges the resulting errors.
	% 
	% outcome = multiUWerrTexp(funhandle, inData, labels)
	% - first arg is a handle to a function accepting a vector made by N elements
	% - inData is a cell-array of N column vectors containing the datalists
	% - third arg is a cell-array of N labels used to list 'ensembles' entering
	% - optional fourth arg is an array of tauexp to feed UWerrTexp.
	%     This has the same structure as labels, i.e. for each ensemble only its first entry is read.
	
	% identify contributing 'ensembles'
	ensembles = unique(labels);
	nens = length(ensembles);
	arglists = {};
	for iens=1:nens
		% which of the input params enter ensemble 'iens' ?
		arglists{iens} = strcmp(labels, ensembles{iens});
	endfor
	
	% check that all datalists for an ensemble have the same size
	ensSize=zeros(1,nens);
	for iens=1:nens
		tEnsSize=[];
		for idata=1:length(labels)
			if arglists{iens}(idata)
				tEnsSize=[tEnsSize length(inData{idata})];
			endif
		endfor
		if length(unique(tEnsSize))>1
			error(['Data from ensemble ''' ensembles{iens} ...
				''' have contradictory sizes (' num2str(tEnsSize) ').']);
		else
			ensSize(iens)=tEnsSize(1);
		endif
	endfor
	
	% print some debug-info
	printf('Ensemble groupings:\n');
	for iens=1:nens
		disp(['  Ensemble ''' sprintf('%4s',ensembles{iens}) ''' (datasize: ' ...
			sprintf('%5u',ensSize(iens)) '). inputs: ' num2str(find(arglists{iens})) ' ']);
	endfor
	
	% now for each ensemble build the proper fhandle and everything
	for iens=1:nens
		targlist = arglists{iens};
		% create the vector with the fixed inputs
		fixpositions = find(1-targlist);
		fIn=[];
		for ivar = 1:length(fixpositions)
			fIn = [fIn mean(inData{fixpositions(ivar)})];
		endfor
		% create the vector with the variable inputs
		varpositions = find(targlist);
		vIn=[];
		for ivar = 1:length(varpositions)
			vIn = [vIn inData{varpositions(ivar)}];
		endfor
		% tparm(1) is left to its default = 1.5 - one can add support for setting it
		% from outside, similar as tauexp below, and the same for all other parms(*)
		tparm(1)=1.5;
		% also extract this ensemble's tauexp from the provided tauExpList, if any
		ttauexp=0;
		if exist('tauExpList','var') && ~isempty(tauExpList)
			ite=tauExpList(varpositions);
			if ~isempty(ite)
				ttauexp=ite(1);
			endif
		endif
		tparm(2)=ttauexp;
		% create the ensemble-restricted fhandle
		partialFun = @(x) call_and_replace(funhandle, targlist, fIn, x);
		% place the call for this ensemble
		[partialVal(iens), partialErr(iens), ~, partialTauint(iens)] = UWerrTexp(vIn, tparm, [], 0, partialFun);
	endfor

	% now combine the errors from the various ensembles into 'err'
	% and (not really necessary, but...) average the values into 'val'
	val=mean(partialVal);
	err=sqrt(sum(partialErr.^2));
	tauintlist = partialTauint;
	
endfunction

function car_result = call_and_replace(funhandle, varglist, fixInputs, varInputs)
	% this accepts:
	%  - a function handle accepting N inputs
	%  - an arglist vector with N entries, of which Nf are fixed and Nv are variable. N=Nf+Nv
	%  - a list of the Nf fixed ones that will be put at the zeros' positions in the full call
	%  - a list of the Nv variable ones that will be put at the ones' positions in the full call
	% and invokes the fun handle after having built the full arguments
	vpositions = find(varglist);
	fpositions = find(1-varglist);
	% reconstruct the full inputs to funhandle
	aInputs = zeros(1,length(varglist));
	for ivar=1:length(fpositions);
		aInputs(fpositions(ivar))=fixInputs(ivar);
	endfor
	for ivar=1:length(vpositions);
		aInputs(vpositions(ivar))=varInputs(ivar);
	endfor
	% place the call
	car_result = funhandle(aInputs);
endfunction
