% Scale-setting through F_pi, 'main' code.
%
% This script load the data, prepares some other information,
% and uses some function to do all the business of (1) locating
% the value of kappa_physical by looking at m_pi/f_pi, and then
% (2) extrapolating (a*f_pi) to this value to get the lattice
% spacing in fermi.
% [Note: cell-arrays and other nice tricks are used.]


% Settings: ensembles' properties
% file location, etc
dataDir='Ensembles/';
ensNames={'E5g'    'F6'      'F7'     'G8'};
fileSuffix='.dat';
% numeric values
kappa   =[0.136250  0.136350  0.136380  0.136417];
tauexp  =[8.35      16.70     16.70     24.66    ];
% this is m_pi/f_pi in our world (=physical point)
ratio_phys = (134.8/130.4)^2; % (Mev/Mev)^2.

% Load all data tables from their files.
% Each file has two columns, resp. a*m_pi and a*f_pi.
% All loaded data will be put into a cell-array
%   (due to the different number of lines, regular array would be impractical)
ensData = cell(1,length(ensNames));
for iens=1:length(ensNames)
	filename = [dataDir ensNames{iens} fileSuffix];
	ensData{iens} = load('-ascii',filename);
	% ensData{iens}(:,1) is a*m_pi, ensData{iens}(:,2) is a*f_pi
endfor

% Now some (complicated) function, find_spacing(...), will be called to
% obtain the lattice spacing.
% A function that in principle accepts as inputs - besides other things - the 4*2 measured
% primary observables (as numbers) and returns that one number, 'a' in fermi.
% This function, if properly built, would then be subject to adequate calls to
% UWerrTexp, roughly: [spa_val, spa_err]=UWerr(inputs,...,...,...,find_spacing,...)
% But there is the additional complication that each pair of inputs actually
% belongs to a different ensemble (luckily completely independent from others) !
% This requires as many separate calls to UWerrTexp as there are ensembles,
% and a final error that will be the combination (in quadrature)
% of the ensemble-specific errors.

% FIRST TEST. 
% As a test (recommended before upgrading to the UWerr-level calls), invoke
% this function in a 'scalar' way, that is, all inputs are numbers.
% * those mean values are computed into an array with [Mpi_1 Fpi_1 Mpi_2 Fpi_2 , ... ]
meanVals=zeros(1,2*length(ensNames));
for iens=1:length(ensNames)
	meanVals(2*iens-1:2*iens) = mean(ensData{iens}); % both Mpi and Fpi at once
endfor
scalSpacing = find_spacing(meanVals,kappa,ratio_phys);

% SECOND METHOD.
% An ad-hoc repeated call to UWerrTexp, one by ensemble. This is a quick solution
% that involves less code writing and less abstract manipulation of function handles and such,
% but is hardly reusable in new analysis in the future.
% The 'partial' errors on the spacing coming from the various calls will be collected
% into spa_err2(1--4) during the various calls.
%
% Here, for each of the four UWerrTexp calls, the plots are shown, and execution
% is suspended with 'pause' (i.e. until the user presses Enter to resume).
% This to stress that inspection of the plots is very recommended with each
% usage of UWerrTexp !
%
spa_val2 = zeros(1,4);
spa_err2 = zeros(1,4);
spa_tauints2 = zeros(1,4);
parm=[1.5 0]; % the second entry will be set to the current ensemble's tauexp
%
% First Ensemble: build a fun-handle where exactly the first and second inputs are 'left free'
% (the first and second entries in the vector 'MF_pairs' entering find_spacing, that is).
spa_find_1 = @(x) find_spacing([x(1) x(2) meanVals(3:end)], kappa, ratio_phys);
parm(2)=tauexp(1);
[spa_val2(1), spa_err2(1), ~, spa_tauints2(1)] = UWerrTexp(ensData{1},parm,[],[],spa_find_1);
% (the '~' placeholder allows to retrieve only return values num. 1, 2 and 4)
disp(['This was ' ensNames{1} '. Press Enter to continue after inspection of the plots!']);
pause
%
% Second Ensemble
spa_find_2 = @(x) find_spacing([meanVals(1:2) x(1) x(2) meanVals(5:end)], kappa, ratio_phys);
parm(2)=tauexp(2);
[spa_val2(2), spa_err2(2), ~, spa_tauints2(2)] = UWerrTexp(ensData{2},parm,[],[],spa_find_2);
disp(['This was ' ensNames{2} '. Press Enter to continue after inspection of the plots!']);
pause
%
% Third Ensemble
spa_find_3 = @(x) find_spacing([meanVals(1:4) x(1) x(2) meanVals(7:end)], kappa, ratio_phys);
parm(2)=tauexp(3);
[spa_val2(3), spa_err2(3), ~, spa_tauints2(3)] = UWerrTexp(ensData{3},parm,[],[],spa_find_3);
disp(['This was ' ensNames{3} '. Press Enter to continue after inspection of the plots!']);
pause
%
% Fourth Ensemble
spa_find_4 = @(x) find_spacing([meanVals(1:6) x(1) x(2)], kappa, ratio_phys);
parm(2)=tauexp(4);
[spa_val2(4), spa_err2(4), ~, spa_tauints2(4)] = UWerrTexp(ensData{4},parm,[],[],spa_find_4);
disp(['This was ' ensNames{4} '. Press Enter to continue after inspection of the plots!']);
pause
% The above repetition of the code should be made more automatised and less redundant;
% it all boils down to creating the right "spa_find_#" function handles call after call.
% One can play a little with that and come up with a solution, which more or less
% is generalised in the following ...

% THIRD METHOD
% Call multiUWerrTexp to let it handle multiple calls to UWerrTexp (one per ensemble).
% First prepare the inData in the proper format and its corresponding labels descriptor.
indata=cell(1,2*length(ensNames));
labels = indata;
tauexplist = zeros(1,2*length(ensNames));
for iens=1:length(ensNames)
	% iens*2-1 is a*m_pi, iens*2 is a*f_pi
	indata{iens*2-1} = ensData{iens}(:,1);
	indata{iens*2} = ensData{iens}(:,2);
	labels{iens*2-1}=ensNames{iens};
	labels{iens*2}=ensNames{iens};
	tauexplist(iens*2-1)=tauexp(iens); % and that's enough for tauexp
endfor
% now the call to multiUWerrTexp. Prepare the adequate function handle to find_spacing and use it.
spacingfinder = @(x) find_spacing(x,kappa,ratio_phys);
[spa_val3, spa_err3, spa_tauints3] = multiUWerrTexp(spacingfinder, indata, labels, tauexplist);
% The advantage of this is that the function 'multiUWerrTexp' is in principle written once for
% all multi-ensemble analysis that you can conceive in the future.

% FINALLY, present all results
% 1. from the scalar test-call
printf('\n');
printf('   * The lattice spacing resulting from the ''scalar'' call is: \n\t%.6f fermi.\n',scalSpacing);
% 2. from the ad hoc (by hand) multiple calls to UWerrTexp. Must still combine things:
spa_final_err2 = sqrt(sum(spa_err2.^2)); % independent ensembles --> sum errors in quadrature
spa_final_val2 = mean(spa_val2); % hopefully this is redundant!
printf('   * The ''ad-hoc'' multi ensemble procedure yields a spacing of:\n\t%.6f +/- %.6f fermi.\n',...
	spa_final_val2, spa_final_err2);
for iens=1:length(ensNames)
	printf('\t  [tauInt(%3s): %.2f]\n', ensNames{iens}, spa_tauints2(iens));
endfor
% 3. from the multiUWerrTexp.
printf('   * The multiUWerrTexp procedure yields a spacing of:\n\t%.6f +/- %.6f fermi.\n',...
	spa_val3, spa_err3);
for iens=1:length(ensNames)
	printf('\t  [tauInt(%3s): %.2f]\n', ensNames{iens}, spa_tauints3(iens));
endfor
% bye.
printf('\n');
